require 'nobrainer_geospatial/version'
require 'nobrainer_geospatial/types/geo_point'
require 'nobrainer_geospatial/types/geo_polygon'

module NoBrainer
  class GeoResultSet
    attr_accessor :rql, :klass

    def initialize(klass, rql)
      self.rql = rql
      self.klass = klass
    end

    def to_a
      NoBrainer.run { self.rql }.to_a.map {|r| new_from_db(r) }
    end

    def all
      to_a
    end

    def first
      new_from_db NoBrainer.run { self.rql }.first
    end

    def each(&block)
      NoBrainer.run { self.rql }.each do |result|
        block.call(new_from_db result)
      end
    end

    def count
      NoBrainer.run { self.rql.count }
    end

    def where(args = nil, &block)
      if block_given?
        GeoResultSet.new(self.klass, self.rql.filter(block))
      else
        GeoResultSet.new(self.klass, self.rql.filter(args))
      end
    end

    def limit(n)
      GeoResultSet.new(self.klass, self.rql.limit(n))
    end

    def skip(n)
      GeoResultSet.new(self.klass, self.rql.skip(n))
    end

    def order_by(args, order = :asc)

      conditions = []

      raise "Invalid sort order: #{order}" unless order == :asc || order == :desc

      if args.class == Array
        conditions = args.map do |arg|
          if order == :asc
            RethinkDB::RQL.new.asc(arg)
          else
            RethinkDB::RQL.new.desc(arg)
          end
        end
      else
        if order == :asc
          conditions = RethinkDB::RQL.new.asc(args)
        else
          conditions = RethinkDB::RQL.new.desc(args)
        end
      end

      GeoResultSet.new(self.klass, self.rql.order_by(*conditions))
    end

    def [](index)
      to_a()[index]
    end

    def to_rql
      self.rql
    end

    private
    def new_from_db(result)
      obj = klass.new_from_db(result)
      if result['_distance']
        obj.singleton_class.class_eval do
          attr_accessor :distance
        end
        obj.distance = result['_distance']
      end
      return obj
    end
  end
end

module NoBrainer
  module GeoQueries

    def self.included base
      base.extend ClassMethods
    end

    module ClassMethods
      def nearest(point, options)
        # if we get an array convert to RQL
        if point.class == Array
          point = RethinkDB::RQL.new.point(*point)
        end
        NoBrainer::GeoResultSet.new(self, self.rql_table.get_nearest(point, options).map {|result| result['doc'].merge({'_distance' => result['dist']})})
      end

      def intersecting(polygon, options)
        if polygon.class == Array
          polygon = RethinkDB::RQL.new.polygon(*polygon)
        end
        NoBrainer::GeoResultSet.new(self, self.rql_table.get_intersecting(polygon, options))
      end
    end

  end
end